// -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
//
// Copyright 2015-2020, Julian Catchen <jcatchen@illinois.edu>
//
// This file is part of Chromonomer.
//
// Chromonomer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Chromonomer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Chromonomer.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef __CHROMONOMER_H__
#define __CHROMONOMER_H__

#include <getopt.h> // Process command-line options
#include <dirent.h> // Open/Read contents of a directory
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iomanip>
using std::setw;
using std::setprecision;
using std::fixed;
#include <sstream>
using std::stringstream;
#include <map>
using std::map;
#include <set>
using std::set;
#include <stack>
using std::stack;
#include <queue>
using std::queue;
#include <list>
using std::list;
#include <forward_list>
using std::forward_list;
#include <iterator>
using std::next;
using std::exception;

#include "constants.h"
#include "scaffold.h"
#include "linkage_map.h"
#include "marker.h"
#include "input.h"
#include "BamI.h"
#include "SamI.h"
#include "FastaI.h"
#include "gzFastaI.h"
#include "log_utils.h"
#include "splits.h"
#include "synteny.h"

int      summarize(map<string, list<Node> > &, map<string, Marker> &, int, char**);

int      prune_map_nodes(string, list<Node> &, map<string, Marker> &);
int      resolve_conflicts(string, list<Node> &, map<string, vector<ScafComp>> &, map<string, Marker> &, map<string, Marker> &, int &, int &, int &);
int      resolve_interlg_conflicts(map<string, Marker> &, map<string, vector<ScafComp>> &, map<string, Marker> &, int &, int &, int &);
int      remove_conflict_markers(map<string, Marker> &, map<string, Marker> &);
int      generate_scaffold_list(map<string, Marker> &, map<string, vector<ScafComp>> &, map<string, Scaffold> &);
int      determine_scaffold_orientation(map<string, Scaffold> &);
ori_type recheck_scaffold_orientation(vector<Marker> &);
int      prune_outoforder_markers(map<string, Scaffold> &, map<string, Marker> &);
int      prune_conflicted_markers(map<string, Scaffold> &, map<string, Marker> &);
int      find_optimal_marker_set(vector<Marker> &, ori_type);
int      markers_in_path(vector<Marker> &, vector<int> &, vector<int> &, ori_type);

int    build_linkage_map(map<string, Scaffold> &, map<string, list<Node> > &);
int    traverse(list<Node> &, ofstream &, bool);
int    span_scaffolds_across_nodes(list<Node> &);
string find_common_scaffolds(Node &, Node &);
Node   extract_node(Node &, Node &, string);

int    parse_lg_definitions(string, map<string, Marker> &);
int    parse_sam_alignments(string, map<string, Marker> &, map<string, Marker> &);
int    prune_unaligned_markers(map<string, Marker> &);
int    prune_misaligned_markers(map<string, Marker> &);
int    parse_agp_file(string, map<string, vector<ScafComp>> &);

int    prune_markers_by_map_node(map<string, vector<ScafComp>> &, map<string, Scaffold> &, map<string, Marker> &, map<string, Marker> &);
int    break_scaffolds_by_marker_cluster(map<string, vector<ScafComp>> &, map<string, Scaffold> &, map<string, Marker> &, map<string, Marker> &, int &, int &, int&);
int    break_scaffolds_by_map_cluster(map<string, vector<ScafComp>> &, map<string, Scaffold> &, map<string, Marker> &, map<string, Marker> &, int &, int &, int &);

int    define_manhattan_dist_cluster(forward_list<Marker> &, vector<Cluster> &, ori_type);

int    parse_depth_of_coverage(string, map<string, vector<DepthWin>> &);
int    identify_depth_coverage_breakpoints(map<string, vector<DepthWin>> &, map<string, vector<ScafComp>> &);
int    insert_depth_coverage_breakpoints(map<string, vector<ScafComp>> &, map<string, vector<ScafComp>> &);

int    write_map(string, string, list<Node> &);
int    write_scaffold_map(string, map<string, list<Node>> &);
int    write_scaffold_agp(string, map<string, vector<ScafComp>> &);
int    write_lg_agp(string, map<string, list<Node> > &, map<string, vector<ScafComp>> &, map<string, Marker> &);
int    write_promising_scaffolds(string, map<string, vector<ScafComp>> &, map<string, Marker> &, set<string> &);
int    translate_fasta(string, string, map<string, list<Node> > &, map<string, vector<ScafComp>> &);
int    write_lg_php(string, string, list<Node> &, string);
int    dump_php_node(Node, string, int &, ofstream &);
int    write_lg_json(string, string, list<Node> &, string);
int    write_final_linkage_map(string, map<string, list<Node>> &);
int    write_fasta(FILE *, Seq *);
int    write_fasta(ofstream &, string &, string &);

int    rev_complement(string &);
bool   compare_marker_bp(Marker, Marker);
bool   compare_marker_bp_reverse(Marker, Marker);
bool   compare_marker_cM_bp(Marker, Marker);
bool   compare_marker_cM_bp_reverse(Marker, Marker);
bool   compare_node_cM(Node, Node);
bool   compare_problem_scaffolds(ProbScaf, ProbScaf);
int    load_marker_list(string, set<string> &);

void   help( void );
void   version( void );
int    parse_command_line(int, char**);

#endif // __CHROMONOMER_H__
